/*-----------------------------*/
/*      pwm.cpp                */
/*     class Pwm               */
/* 2003.12.16 MIRS0204         */
/*-----------------------------*/
/* gcc option: -lm             */

#include <iostream.h>
#include <math.h>   // abs()
#include <stdlib.h> // exit()
#include <unistd.h> // close()
#include <fcntl.h>  // open()

#include "pwm_enc.h"
#include "pwm.h"

Pwm::Pwm(void)
{
	if((fd_rtf4 = open("/dev/rtf4", O_WRONLY)) < 0){
		cerr << "ERROR: /dev/rtf4" << endl;
		exit(EXIT_FAILURE);
	}
	if((fd_rtf5 = open("/dev/rtf5", O_WRONLY)) < 0){
		cerr << "ERROR: /dev/rtf5" << endl;
		exit(EXIT_FAILURE);
	}
	

	velocity = 0;
	//direction = 0;
}

Pwm::~Pwm(void)
{
	close(fd_rtf4);
	close(fd_rtf5);
}

void Pwm::setPwm(int left_duty, int right_duty)
{
	int left_data, right_data;
	if(left_duty > 100) left_duty=100;
	if(left_duty < -100) left_duty=-100;
	if(right_duty > 100) right_duty=100;
	if(right_duty < -100) right_duty=-100;

// LEFT
	int left_duty2, right_duty2;
	if(left_duty < 0){
		left_duty2 = -101-left_duty;
	}else{
		left_duty2 = left_duty;
	}
	left_data = abs(left_duty2) * 127 / 100;
	if(left_duty2 >= 0){
		left_data += 128; 
	}
// RIGHT
	if(right_duty > 0){
		right_duty2 = right_duty -101;
	}else if(right_duty == 0){
		right_duty2 = 0;
	}else{
		right_duty2 = -right_duty;
	}
	right_data = abs(right_duty2) * 127 /100;
	if(right_duty2 >= 0){ 
		right_data += 128;
	}
// WRITE
	data_write(left_data, right_data, 0, DUTY_MODE);
	
}

int Pwm::stopPwm(void)
{
	data_write(128,128,0,DUTY_MODE);
	return 0;
}

// private function
void Pwm::data_write(const int left, const int right, const int tup, const int mode)
{
	my_msg_struct msg;
	msg.command = START_TASK;
	msg.task = 2;
	msg.period = 10000;
	if(write(fd_rtf5,&msg,sizeof(msg)) < 0){
		cerr << "can't send a command to RT-task" << endl;
		exit(EXIT_FAILURE);
	}

	pwmd.left_speed = left;
	pwmd.right_speed = right;
	pwmd.tup = tup;
	pwmd.mode = mode;
#ifdef DEBUG
	cout << "PwmLeft: " << left << "   PwmRight:" << right << '\n';
#endif
	if(write(fd_rtf4, &pwmd, sizeof(pwmd)) < 0){
		cerr << "ERROR: write /dev/rtf4" << endl;
	}
}
