/*********************************************/
/*      IRS  Device Driver Program           */
/*      irs.c                                */
/*                 developed by Keisuke Muto */
/*                     ver 1.0  Dec.    2003 */
/*********************************************/

#define __KERNEL__  
#define MODULE

#include <linux/errno.h>     
#include <linux/fs.h>
#include <linux/module.h>
#include <linux/ioport.h>
#include <linux/ioctl.h>
#include <asm/io.h>
#include <asm/segment.h>
#include <asm/uaccess.h>

#define IRS_MAJOR 62
#define IRS_PORT_ADDR 0x0150
#define IRS_PORT_ADDR_S 0x0152
#define IRS_RANGE 2

static unsigned int busy = 0;


/*=================================================/
/              irs open                            /
/=================================================*/

static int irs_open(struct inode *inode, struct file *filp){
  if (busy) return - EBUSY;        
  busy = 1;

	if (check_region(IRS_PORT_ADDR, IRS_RANGE) < 0){
			printk("ERROR:irs_address has been used\n");
			return -EBUSY;
	}

  MOD_INC_USE_COUNT;

	request_region(IRS_PORT_ADDR, IRS_RANGE, "irs");
			
  return 0;
}


/*=================================================/
/              irs ioctl                            /
/=================================================*/

static int irs_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg){
  
  int kari;
  
	if(cmd==10){
		kari=inw(IRS_PORT_ADDR);
    // printk("IRS=%x form irs.o",kari);
	}
	else{
		outb(cmd,IRS_PORT_ADDR_S);
		kari=0;
	}
	put_user(kari,(int *)arg);
	return 0;
}


/*=================================================/
/              irs release                         /
/=================================================*/

static int irs_release(struct inode *inode, struct file *filp){
  busy = 0;
  MOD_DEC_USE_COUNT;
  release_region(IRS_PORT_ADDR,IRS_RANGE);
  return 0;
}


/*=================================================/
/      struct file_operations                      /
/=================================================*/
static struct file_operations irs_fops = {
   owner:              THIS_MODULE,
   read:               NULL,
   write:              NULL,
   ioctl:              irs_ioctl,
   open:               irs_open,
   release:            irs_release,
 };

/*=================================================/
/              init_module                         /
/=================================================*/

int init_module (){
  int i;
  i = register_chrdev (IRS_MAJOR, "irs", &irs_fops);
  if (i){
			printk("ERROR:register_chrdev don't about irs\n");
      return -EBUSY;
  }
  return 0;
} 
/*=================================================/
/              cleanup_module                      /
/=================================================*/
void cleanup_module (void){
  int i;
  i = unregister_chrdev (IRS_MAJOR,"irs");
	if(i){
	printk("ERROR:unregister_chrdev don't about irs\n");
  }

}
