/*-----------------------------*/
/*      enc.cpp                */
/*     class Enc               */
/* 2003.11.18 MIRS0204         */
/*-----------------------------*/


#include <iostream.h>
#include <stdlib.h> // exit(1)
#include <unistd.h> // close(1)
#include <fcntl.h>  // open(2)
#include <sys/time.h>
#include <sys/types.h>

#include <rtl_fifo.h>
#include <rtl_time.h>
#include <rtl_debug.h>

#include "pwm_enc.h"
#include "enc.h"

Enc::Enc(void)
{
  if ((fd = open("/dev/rtf3",O_RDONLY)) < 0) {
		cerr << "ERROR opening /dev/rtf3" << endl;
		exit(EXIT_FAILURE);
  }
	
	init_flg = 0;

 	l_distance = 0;
  	r_distance = 0;
  	l_speed = 0;
  	r_speed = 0;
	l_now = 0;
	r_now = 0;
	l_old = 0;
	r_old = 0;
}

Enc::~Enc(void)
{
  close(fd);
}

int Enc::getValue(void)
{
	int fd5;
	my_msg_struct msg;
	if((fd5 = open("/dev/rtf5", O_WRONLY)) < 0){
		cerr << "can't open /dev/rtf5" << endl;
		exit(EXIT_FAILURE);
	}
	msg.command = START_TASK;
	msg.task = 2;
	msg.period = 10000;
	if(write(fd5,&msg, sizeof(msg)) < 0){
		cerr << "can't send a command to RT-task" << endl;
		exit(EXIT_FAILURE);
	}
	close(fd5);


	fd_set rfds;
	struct timeval tv;
	count_data count;
	int retval, retval2;
	
	
	FD_ZERO(&rfds);
	FD_SET(fd, &rfds);
	tv.tv_sec = 0;
	tv.tv_usec = 100000;
	
	retval = select(FD_SETSIZE, &rfds, NULL, NULL, &tv);
	if (retval > 0) {
		while(1){
			read(fd, &count, sizeof(count));
			retval2 = select(FD_SETSIZE, &rfds, NULL, NULL, &tv);
			if (retval2 == 0) {
				break;
			}
		}
	}
	
	int div;
	
	l_now = count.left_sum  * DIAMETER * PI / ( CPR * GEAR);
	r_now = count.right_sum * DIAMETER * PI / ( CPR * GEAR);
	if(init_flg == 0){
		l_old = l_now;
		r_old = r_now;
		init_flg = 1;
	}
	div = l_now - l_old;
	l_distance += div;
	l_old = l_now;

	div = r_now - r_old;
	r_distance += div;
	r_old = r_now;

//	l_distance = count.left_sum;
//	r_distance = count.right_sum;

	l_speed = count.left_count  * DIAMETER * PI / (CPR * GEAR);
	r_speed = count.right_count * DIAMETER * PI / (CPR * GEAR);
	
	cout << count.left_sum << "  " << count.right_sum << endl << endl;

	return 0;
}

int Enc::getDistance(position p)
{
	int rv;
	
	switch(p){
	case left:
		rv = -l_distance;
		break;
	case right:
		rv = r_distance;
		break;
	default:
		rv = -1.;
		break;
	}
	return rv;
}

int Enc::getSpeed(position p)
{
	int rv;
	
	switch(p){
	case left:
		rv = -l_speed;
		break;
	case right:
		rv = r_speed;
		break;
	default:
		rv = -1.;
		break;
	}
	return rv;
}

void Enc::reset(void)
{
	l_distance = 0;
	r_distance = 0;
	
}
