/*-----------------------------*/
/*      pwm_enc.h                 */
/*                             */
/* 2003.11.20 MIRS0204         */
/*-----------------------------*/

#ifndef _PWM_ENC_H
#define _PWM_ENC_H


/* For the task control */
#define COMMAND_FIFO  5
#define START_TASK    1   
#define STOP_TASK     2  
#define RESTART_TASK  3  

/* For Rotary Encoder */
/* Data of tire & motor */

#define DIAMETER  65      /* diameter of the tire [mm] */
#define PI         3.141592
#define GEAR	16
#define PERI       0.02  /* execution period of RT-Task [sec] */
#define CPR         100  /* the number of counts a revolution */
#define INTERVAL     22  /* the intarval between the tires */

#define ENC 167          /* the circumference of encoder */

/* For PWM */
#define KEEP_SPEED -999   
#define DUTY_MODE   123
#define STOP_MODE   456

/* define the I/O ports */
#define PWM_LEFT_WRITE   0x0110
#define PWM_RIGHT_WRITE  0x0112
#define LEFT_COUNTER     0x0120
#define RIGHT_COUNTER    0x0122

struct my_msg_struct{
	int command;     /* command to control RT-task */
	int task;        /* number of the RT-task */
	int period;      /* execution period of the RT-task */
};

struct count_data{
	int left_count;        /* pulses of left counter */
	int right_count;       /* pulses of right counter */
	int left_sum;    /* sum of puslses of left counter */
	int right_sum;    /* sum of puslses of right counter */
	int br;
};

struct send_data{
	double aomegar;
	double aomegal;
	double omegar;
	double omegal;
	double tn;
	int br;
};


struct pwm_speed_data{
  int left_speed;     /* velocity of left wheel */
  int right_speed;     /* velocity of right wheel */
  int tup;
  int mode;
};


/* define position type */
typedef enum {
	left,
	right,
	front,
	rear
} position;


#endif
